<?php

/*
 *
 * action functions make themselves known and take effect when called as hooks from connect
 *
 * @params $op
 * 'describe' -> return array of info for config UI
 * 'requires' -> describe parent & child data requirements for config UI
 * 'prepare_data' -> make changes to parent/child values before 'insert' is called
 * 'validate'   -> evaluate child content (NB $child = (array) $form_data, not (obj) $child)
 * 'form_alter' -> allows actions to affect the child node form when before it's displayed
 * 'insert'  -> handle changes to $target nodes
 * 'update'  -> handle changes to $target nodes
 * 'delete'  -> handle changes to $target nodes
 * 'display' -> node is being displayed
 * 'status'  -> reports on status of child node interaction
 *
 *  @return
 *  if $op='display', an array( 'name' => string, 'data' => string )
 *  if $op='status', an array( 'message' => string, 'show_form' => boolean )
 *  if $op='validate', array( 'status' => boolean, 'message' => string )
 *  if $op='form_alter', object $child = the revised form fields
 */


/**
 * Mandatory action, handles basic housekeeping
 */
function connect_action_basic($op='', &$parent=NULL, &$child=NULL, $target='parent') {
  if ( $target == 'child' ) {
      switch ($op) {
        // record fact of current user's participation
        case 'insert' :
          $_SESSION['connect_action_basic_'.$parent->nid] = TRUE;
          $_SESSION['connect_action_thanks_'.$parent->nid] = TRUE;
          break;
      }

  } elseif ($target == 'parent') {
    switch ($op) {
      case 'describe' :
        $return = array();
        $return['name']  = 'connect_action_basic';
        $return['title'] = 'Basic settings';
        return $return;
        break;

      case 'requires' :
        $return = array();
        $default = connect_node_options( $parent->nid, 'is_live' );
        $default = $default ? $default : 'no';
        $return['variables']['is_live'] = array(
          '#type'  => 'radios',
          '#title' => t('Is this campaign ready to go live?'),
          '#description' => t('This is only relevant if you are sending faxes and emails. \'No\' means that no emails, faxes, etc. will be sent; you will instead see a display of the message(s) that would have been sent.'),
          '#options' => array( 'yes' => 'Yes', 'no' => 'No' ),
          '#default_value' => $default,
          '#required' => TRUE,
        );

        $types = connect_participant_types_options();
        $types[0] = '';
        asort($types);
        $return['variables']['participant_type'] = array(
          '#type'  => 'select',
          '#title' => 'Participant node type',
          '#options' => $types,
          '#default_value' => connect_node_options( $parent->nid, 'participant_type' ),
          '#required' => TRUE,
        );

        $return['variables']['participant_title'] = array(
          '#type'  => 'textfield',
          '#title' => 'Title for the participants',
          '#default_value' => connect_node_options( $parent->nid, 'participant_title' ),
          '#required' => TRUE,
        );

        $return['variables']['call_to_action'] = array(
          '#type'  => 'textfield',
          '#title' => 'Call to action',
          '#default_value' => connect_node_options( $parent->nid, 'call_to_action' ),
          '#required' => TRUE,
        );

        $return['variables']['thankyou'] = array(
          '#type'  => 'textarea',
          '#size' => 4,
          '#title' => 'Thank-you message',
          '#default_value' => connect_node_options( $parent->nid, 'thankyou' ),
          '#required' => TRUE,
        );

        return $return;
        break;

    case 'status' :
      // display 'thank you message'
      if ( isset($_SESSION['connect_action_thanks_'.$parent->nid]) ) {
        unset($_SESSION['connect_action_thanks_'.$parent->nid]);
        $message = connect_node_options( $parent->nid, 'thankyou' );
        return array(
          'status' => $message,
          'show_form' => FALSE,
        );
      }
    }
  }
}


/**
 * Adds a CSS-based display to the parent node showing how many of the target number of participants have participated
 */
function connect_action_display_progress($op='', &$parent=NULL, &$child=NULL, $target='parent') {
  switch($op) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_display_progress';
      $return['title'] = 'Display progress bar';
      $return['desc']  = 'Adds a CSS-based display showing how many of the target number have participated.';
      return $return;
      break;

    case 'requires' :
      $return = array();
      $return['variables'] = array(
          'display_progress_goal' => array(
            '#type'  => 'textfield',
            '#size' => 20,
            '#title' => 'Target no. of participants',
            '#default_value' => connect_node_options( $parent->nid, 'display_progress_goal' ),
            '#required' => TRUE,
          ),
      );
      return $return;
      break;

    case 'display' :
      if ( $target != 'parent' || $parent == NULL ) {
        return;
      }

      $done  = connect_participant_count($parent);
      $title = connect_node_options( $parent->nid, 'participant_title' );
      $goal  = connect_node_options( $parent->nid, 'display_progress_goal' );
      if ($goal > 0) {
        $pct  = round($done/$goal, 2) * 100;
        $pct_display = $pct > 100 ? 100 : $pct;
      } else {
        $goal = 0;
        $pct  = 0;
        $pct_display = 0;
      }
      $return['name'] = 'display_progress';
      $return['data'] = theme('connect_display_progress', $goal, $pct, $pct_display, $title);
      return $return;
  }
}

function theme_connect_display_progress($goal, $pct, $pct_display, $title) {
  $progress_bar = <<<EOT
<div>
<div id="connect-progress-report">$goal $title</div>
<div id="connect-progress-border">
<div id="connect-progress-bar" style="width: $pct_display%;">
$pct%
</div>
</div>
</div>
EOT;
  return $progress_bar;
}


/**
 * Enforces a one person, one vote policy based on a unique identifier
 */
function connect_action_voteonce($op='', &$parent=NULL, &$child=NULL, $target='child') {

  $message = connect_node_options( $parent->nid, 'voteonce_message' );
  $message = $message ? $message : 'You have already participated.';

  switch($op) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_voteonce';
      $return['title'] = 'One vote per person';
      $return['desc']  = 'Prevent the same person from participating more than once.';
      return $return;
      break;

    case 'requires' :
      $return = array();
      $return['child'] = array(
        'voteonce_identifier' => 'Unique identifier (i.e., full name, membership #, or email address)',
      );
      $return['variables'] = array(
        'voteonce_message' => array(
          '#type'  => 'textfield',
          '#title' => 'Message to display when someone attempts to participate more than once',
          '#default_value' => $message,
          '#required' => TRUE,
        ),
      );
      return $return;
      break;

    case 'validate' :
      $map  = connect_get_map($parent->nid);
      $test_value  = connect_value( 'voteonce_identifier', $parent, $child, 'child' );

      $fieldname   = $map['voteonce_identifier'];
      $field_keys  = connect_get_field_keys($fieldname);
      $test_db     = _connect_get_cck_db_info($fieldname);
      $test_table  = $test_db['table'];
      $test_column = $test_db['columns'][$field_keys[0]]['column'];

      $sql   = "SELECT count(*) FROM {".$test_table."} t, {connect_data} p WHERE t.nid=p.nid AND t.$test_column = '%s' AND p.pid = %d";
      $count = db_result(db_query($sql, array($test_value, $parent->nid)));
      $already_voted = ($count != 0);

      if ($already_voted) {
        form_set_error('', t($message));
      }
      break;

    case 'status' :
      // if this user has already participated
      if ( isset($_SESSION['connect_action_basic_'.$parent->nid]) ) {
        return array(
          'status' => $message,
          'show_form' => FALSE,
        );
      }
      break;
  }
}


/**
 *  allows participant-generated content to be added to the parent content
 */
function connect_action_content_replace($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_content_replace';
      $return['title'] = 'Content: rewrite';
      $return['desc']  = 'Allows participants to revise the content provided by the parent node.';
      return $return;

    case 'requires' :
      $return = array();
      $return['parent'] = array(
        'data_replace_parent' => 'Content replace: the content that can be rewritten.',
      );
      $return['child']  = array(
        'data_replace_child' => 'Content replace: the participant\'s version.',
      );
      return $return;

   case 'form_alter':
     $map   = connect_get_map($parent->nid);
     $field = $map['data_replace_child'];
     $key   = connect_get_field_keys($field);
     $child[$field][0][$key[0]]['#default_value'] = connect_value( 'data_replace_parent', $parent, $child, 'parent' );
     return $child;
     break;

    case 'prepare_data' :
      $addition = connect_value( 'data_replace_child', $parent, $child, 'child' );
      connect_value( 'data_replace_parent', $parent, $child, 'parent', $addition );
      break;
  }
}


/**
 *  allows participant-generated content to be added to the parent content
 */
function connect_action_content_append($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_content_append';
      $return['title'] = 'Content: append';
      $return['desc']  = 'Allows participants to add their own comments, etc. to the content provided by the parent node.';
      return $return;

    case 'requires' :
      $return = array();
      $return['parent'] = array(
        'data_append_parent' => 'Content append: the content that can be added on to.',
      );
      $return['child']  = array(
        'data_append_child' => 'Content replace: the participant\'s addition.',
      );
      return $return;

    case 'prepare_data' :
      $original = connect_value( 'data_append_parent', $parent, $child, 'parent' );
      $addition = connect_value( 'data_append_child', $parent, $child, 'child' );
      connect_value( 'data_append_parent', $parent, $child, 'parent', "$original\n$addition" );
      break;
  }
}



/**
 *  Saves participant data to CiviCRM
 */

/*
 * TODO: test this out more
 * 
function connect_action_civicrm_participants($op='', &$parent=NULL, &$child=NULL, $target='child') {
  if (!module_exists('civicrm')) {
    return;
  }

  switch( $op ) {
    case 'describe' :
      $return = array();
       $return['name']  = 'connect_action_civicrm_participants';
       $return['title'] = 'CiviCRM: add participants';
       $return['desc']  = 'Saves participant information to CiviCRM.';
      return $return;

    case 'requires' :
      $return = array();

      // child vars
      $return['child']  = array(
        'civicrm_contact_id' => 'CiviCRM contact ID.',
      );

      // store CiviCRM mapping in options rather than child variable map
      // because not all are required
      $child_type = connect_node_options( $parent->nid, 'participant_type' );
      if ( $child_type) {
        $cck_info = _content_type_info();
        $cck_vars = $cck_info['content types'][$child_type]['fields'];
        foreach ($cck_vars as $name=>$field) {
          if ($field['display_settings']['teaser']['format'] != 'hidden' && $field['display_settings']['full']['format'] != "hidden") {
            $cck_options[$name] = $field['widget']['label'];
          }
        }
        $cck_options[''] = '';
        asort($cck_options);

        $return['variables']['civicrm_participants_intro'] = array(
          '#value' => t('Select the participant/child fields that match the given CiviCRM fields. First and last name and email are required.'),
        );
        $crm_vars = connect_civicrm_properties_options();
        foreach($crm_vars as $key=>$name) {
          $return['variables']["civicrm_participants_$key"] = array(
            '#type' => 'select',
            '#title' => $name,
            '#options' => $cck_options,
            '#default_value' => connect_node_options( $parent->nid, "civicrm_participants_$key" ),
          );
        }
      }
      else {
        $return['variables']['civicrm_participants_message'] = array(
          '#value' => t('Please select a participant type and return here to set up your variables.'),
        );
      }
      return $return;

    case 'validate' :
      if ($target != 'child') {
        return;
      }
      $required = array('email'=>'Email address', 'first_name'=>'First name', 'last_name'=>'Last name');
      foreach ($required as $key=>$name) {
        $var  = connect_node_options( $parent->nid, "civicrm_participants_$key" );
        $path = _connect_get_field_path($child, $var);
        if (empty($var) || empty($path)) {
          form_set_error('', t('There is a problem with the parent node configuration. Please contact the site administrator.'));
          return;
        }
        eval('$field_value = $child'. $path .';');
        if ($key == 'email') {
          if (!_connect_valid_email($field_value)) {
            form_set_error($var, "$name must be a valid address.");
          }
        }
        elseif (empty($field_value)) {
          form_set_error($var, "$name is required");
        }
      }
      break;

    case 'insert' :
      if ($target != 'child') {
        return;
      }
      civicrm_initialize();
      require_once('api/v2/Contact.php');
      require_once('api/v2/Group.php');
      require_once('api/v2/GroupContact.php');

      // grab CRM data from child node
      $crm_params = array();
      $options    = connect_node_options($parent->nid);
      foreach ($options as $option=>$field_name) {
        if (!empty($field_name)) {
          $regex = '/^civicrm_participants_(.*)$/';
          if (preg_match($regex, $option, $matches) == 1) {
            $crm  = $matches[1];
            $path = _connect_get_field_path($child, $field_name);
            eval('$field_value = $child'. $path .';');
            if (!empty($field_value)) {
              $crm_params[$crm] = $field_value;
            }
          }
        }
      }

      // update or create contact
      if ( !empty($crm_params) ) {
        $crm_params['contact_type'] = 'Individual';

        var_dump($crm_params);

        $contact = civicrm_contact_add($crm_params);

        $params = array(
          'contact_id' => $contact['contact_id'],
          'return.contact_id' => 1,
          'return.display_name' => 1,
          'return.postal_code' => 1,
        );
        $retrieved = &civicrm_contact_search( $params );
        var_dump($retrieved);
        die();

        if ($result['is_error'] == 0) {
          // save contact ID in cck record
          connect_value( 'civicrm_contact_id', $parent, $child, 'child', $contact['contact_id']);
          node_save($child);

          // find or create group
          $params = array('title' => $parent->title, 'is_active' => 1);
          $groups = civicrm_groups_get($params);

          if (!$groups['is_error']) {
            $keys = array_keys($groups);
            $group_id = $keys[0];
          } else {
            $result = civicrm_group_add($params);
            $group_id = $result['result'];
          }

          // add contact to group
          $params = array('group_id' => $group_id, 'contact_id' => $contact['contact_id']);
          $result = civicrm_group_contact_add($params);
          if ($result['is_error']) {
            watchdog('debug', "connect: $contact[contact_id] not added to group $group_id");
          }
        } else {
          watchdog('debug', 'connect: CiviCRM record not created: '.$participant->nid);
        }
      } else {
        watchdog('debug', 'connect: CiviCRM params empty: '.$participant->nid);
      }
  }
}
*/


/**
 *  send email to a defined target
 */
function connect_action_email_defined($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_email_defined';
      $return['title'] = 'Email (defined target)';
      $return['desc']  = 'Sends email to a specified name and email address.';
      return $return;
      break;

    case 'requires' :
      $return = array();
      $return['variables'] = array(
        'email_to_address' => array (
          '#type'  => 'textfield',
          '#size' => 20,
          '#title' => 'Target email address',
          '#default_value' => connect_node_options( $parent->nid, 'email_to_address' ),
          '#required' => TRUE,
        ),
        'email_to_name' => array (
          '#type'  => 'textfield',
          '#size' => 20,
          '#title' => 'Target name',
          '#default_value' => connect_node_options( $parent->nid, 'email_to_name' ),
          '#required' => TRUE,
        ),
      );

      // send HTML mail?
      if (module_exists('mimemail')) { 
        if (!variable_get('mimemail_alter', 0)) {
          $return['variables']['email_send_html'] = array(
            '#type'  => 'radios',
            '#title' => 'Send HTML email using the mimemail module?',
            '#options' => array( 'yes' => 'Yes', 'no' => 'No' ),
            '#default_value' => connect_node_options( $parent->nid, 'email_send_html' ),
            '#required' => TRUE,
          );
        }
      } else {
        $return['variables']['email_send_html'] = array(
          '#value' => '('. t('If you want to be able to send HTML emails, install the mimemail module.') .')',
        );
      }

      $return['parent'] = array(
        'email_subject' => 'Email subject',
        'email_body'    => 'Email body',
      );
      $return['child']  = array(
        'email_from'   => 'Participant\'s email',
        'email_defined_result' => 'Mail (defined target) success/failure message',
      );
      return $return;
      break;

    case 'validate' :
      if ($target != 'child') {
        return;
      }
      $email = connect_value( 'email_from', $parent, $child, 'child' );
      if ( !valid_email_address($email) ) {
        form_set_error('', 'Please enter a valid email address');
      }
      break;

    case 'insert' :
      if ($target != 'child') {
        return;
      }

      $message->to      = connect_node_options( $parent->nid, 'email_to_address' );
      $message->from    = connect_value( 'email_from', $parent, $child, 'child' );
      $message->subject = connect_value( 'email_subject', $parent, $child, 'parent' );
      $message->body    = connect_value( 'email_body', $parent, $child, 'parent' );

      $active = (connect_node_options($parent->nid, 'is_live') == 'yes');
      $html   = (connect_node_options($parent->nid, 'email_send_html') == 'yes');
      $result = _connect_send_email($message, $html, $active);

      // save result
      $_SESSION['connect_'.$parent->nid.'_email_sent'] = $result ? $message->to : FALSE;
      $result = $result ? "Success" : "Failed";
      connect_value( 'email_defined_result', $parent, $child, 'child', $result);
      node_save($child);
      break;

    case 'status' :
      $message  = '';
      if ( isset($_SESSION['connect_'.$parent->nid.'_direct_email_sent']) ) {
        $message = $_SESSION['connect_'.$parent->nid.'_direct_email_sent'] ? 'Your email message was sent to ' .$_SESSION['connect_'.$parent->nid.'_direct_email_sent'] : 'Sorry, but there was a problem sending the message. The problem will be investigated, and your message will be sent when the problem is resolved.';
        unset($_SESSION['connect_'.$parent->nid.'_direct_email_sent']);
      }
      return array(
        'status' => $message
      );
      break;
  }
}

/**
 *  send email to a target determined on the basis of participant data
 */
function connect_action_email_lookup($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_email_lookup';
      $return['title'] = 'Email (target lookup)';
      $return['desc']  = 'Sends email to a target determined by the participant\'s information.';
      return $return;
      break;

    case 'requires' :
      require_once( drupal_get_path('module','connect') . '/connect_lookup.php');
      $base = array();
      $options = connect_get_lookup_types(TRUE);
      $base['variables'] = array(
        'email_lookup_type' => array(
          '#type'  => 'select',
          '#title' => 'Email target lookup',
          '#options' => $options,
          '#default_value' => connect_node_options( $parent->nid, 'email_lookup_type' ),
          '#required' => TRUE,
        ),
      );
      $base['parent'] = array(
        'email_subject' => 'Email subject',
        'email_body'    => 'Email body',
      );
      $base['child']  = array(
        'email_from'   => 'Participant\'s email',
        'email_lookup_result' => 'Mail (lookup target) success/failure message',
      );

      // send HTML mail?
      if (module_exists('mimemail') && !variable_get('mimemail_alter', 0)) {
        $return['variables']['email_send_html'] = array(
          '#type'  => 'radios',
          '#title' => 'Send HTML email using the mimemail module?',
          '#options' => array( 'yes' => 'Yes', 'no' => 'No' ),
          '#default_value' => connect_node_options( $parent->nid, 'email_send_html' ),
          '#required' => TRUE,
        );
      }

      // add requirements from lookup type
      $lookup = connect_target_lookup('requires', $parent, $child, 'email');
      if (is_array($lookup)) {
        foreach( array('variables','parent','child') as $key ) {
          if (isset($lookup[$key])) {
            $return[$key] = $base[$key]+$lookup[$key];
          } else {
            $return[$key] = $base[$key];
          }
        }
      } else {
        $return = $base;
        drupal_set_message('You have not selected a lookup type. Please do so and then adjust any new settings that are required.');
      }

      return $return;
      break;

    case 'validate' :
      if ($target != 'child') {
        return;
      }
      $email = connect_value( 'email_from', $parent, $child, 'child' );
      if ( !valid_email_address($email) ) {
        form_set_error('', 'Please enter a valid email address');
      }
      // call validation from lookup type
      connect_target_lookup('validate', $parent, $child, 'email');
      break;

    case 'insert' :
      if ($target != 'child') {
        return;
      }

      require_once( drupal_get_path('module','connect') . '/connect_lookup.php');
      $target  = connect_target_lookup( 'lookup', $parent, $child, 'email' );
      if ($target) {
        $message->to      = $target['email'];
        $message->from    = connect_value( 'email_from', $parent, $child, 'child' );
        $message->subject = connect_value( 'email_subject', $parent, $child, 'parent' );
        $message->body    = connect_value( 'email_body', $parent, $child, 'parent' );

        $active = (connect_node_options($parent->nid, 'is_live') == 'yes');
        $html   = (connect_node_options($parent->nid, 'email_send_html') == 'yes');
        $result = _connect_send_email($message, $html, $active);
      } else {
        $result = FALSE;
      }

      // save result
      $_SESSION['connect_'.$parent->nid.'_lookup_email_sent'] = $result ? $target['name'].' ('.$target['email'].')' : FALSE;
      $result = $result ? "Success" : "Failed";
      connect_value( 'email_lookup_result', $parent, $child, 'child', $result );
      node_save($child);
      break;

    case 'status' :
      $message = '';
      if ( isset($_SESSION['connect_'.$parent->nid.'_lookup_email_sent']) ) {
        $message = $_SESSION['connect_'.$parent->nid.'_lookup_email_sent'] ? 'Your email message was sent to ' .$_SESSION['connect_'.$parent->nid.'_lookup_email_sent'] : 'Sorry, but there was a problem sending the message. The problem will be investigated, and your message will be sent when the problem is resolved.';
        unset($_SESSION['connect_'.$parent->nid.'_lookup_email_sent']);
      }
      return array('status' => $message);
      break;
  }
}


/**
 * Sends fax (using the myfax.com service) to a specific name and fax no. defined in the parent node.
 */
function connect_action_myfax_defined($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_myfax_defined';
      $return['title'] = 'MyFax (defined target)';
      $return['desc']  = 'Sends fax (using the myfax.com service) to a specified name and fax no.';
      return $return;
      break;

    case 'requires' :
      $return = array();
      $return['variables']  = array(
        'fax_to_number' => array (
          '#type'  => 'textfield',
          '#size' => 20,
          '#title' => 'Target fax no.',
          '#default_value' => connect_node_options( $parent->nid, 'fax_to_number' ),
          '#required' => TRUE,
        ),
        'fax_to_name' => array (
          '#type'  => 'textfield',
          '#size' => 20,
          '#title' => 'Target name',
          '#default_value' => connect_node_options( $parent->nid, 'fax_to_name' ),
          '#required' => TRUE,
        ),
        'myfax_email' => array(
          '#type'  => 'textfield',
          '#title' => 'MyFax account \'from\' email address',
          '#default_value' => connect_node_options( $parent->nid, 'myfax_email' ),
          '#required' => TRUE,
        ),
        'myfax_password' => array(
          '#type'  => 'textfield',
          '#title' => 'MyFax password',
          '#default_value' => connect_node_options( $parent->nid, 'myfax_password' ),
        ),
      );
      $return['parent'] = array(
        'fax_subject' => 'Fax subject',
        'fax_body'    => 'Fax body',
      );
      $return['child']  = array(
        'fax_direct_result' => 'MyFax (direct) success/failure message',
        'fax_from'   => 'Fax \'from\' name',
      );
      return $return;
      break;

    case 'validate' :
      if ($target != 'child') {
        return;
      }
      $name = connect_value( 'fax_from', $parent, $child, 'child' );
      if ( empty($name) ) {
        form_set_error('', 'Please enter your name.');
      }
      break;

    case 'insert' :
      if ($target != 'child') {
        return;
      }

      $fax = array();
      $fax['from_mail'] = connect_node_options( $parent->nid, 'myfax_email' );
      $fax['to_fax']    = connect_node_options( $parent->nid, 'fax_to_number' );
      $fax['to_name']   = connect_node_options( $parent->nid, 'fax_to_name' );
      $fax['password']  = connect_node_options( $parent->nid, 'myfax_password' );
      $fax['body']      = connect_value( 'fax_body', $parent, $child, 'parent' );
      $fax['subject']   = connect_value( 'fax_subject', $parent, $child, 'parent' );
      $fax['from_name'] = connect_value( 'fax_from', $parent, $child, 'child' );

      $message = connect_myfax_prepare($fax);
      $active  = (connect_node_options($parent->nid, 'is_live') == 'yes');
      $result  = _connect_send_email($message, FALSE, $active);

      // save result
      $_SESSION['connect_'.$parent->nid.'_direct_fax_sent'] = $result;
      $result = $result ? "Success" : "Failed";
      connect_value( 'fax_direct_result', $parent, $child, 'child', $result);
      node_save($child);
      break;

    case 'status' :
      $message  = '';
      if ( isset($_SESSION['connect_'.$parent->nid.'_direct_fax_sent']) ) {
        $message = $_SESSION['connect_'.$parent->nid.'_direct_fax_sent'] ? 'Your fax message was sent!' : 'Sorry, but there was a problem sending the message. The problem will be investigated, and your message will be sent when the problem is resolved.';
        unset($_SESSION['connect_'.$parent->nid.'_direct_fax_sent']);
      }
      return array(
        'status' => $message
      );
      break;
      break;
  }
}

/**
 * Sends fax (using the myfax.com service) to a target determined by the participant\'s information.
 */
function connect_action_myfax_lookup($op='', &$parent=NULL, &$child=NULL, $target='child') {
  switch( $op ) {
    case 'describe' :
      $return = array();
      $return['name']  = 'connect_action_myfax_lookup';
      $return['title'] = 'MyFax (target lookup)';
      $return['desc']  = 'Sends fax (using the myfax.com service) to a target determined by the participant\'s information.';
      return $return;
      break;

    case 'requires' :
      require_once( drupal_get_path('module','connect') . '/connect_lookup.php');
      $return = array();
      $options = connect_get_lookup_types(TRUE);
      $base['variables']  = array(
        'myfax_lookup_type' => array(
          '#type'  => 'select',
          '#title' => 'MyFax target lookup',
          '#options' => $options,
          '#default_value' => connect_node_options( $parent->nid, 'myfax_lookup_type' ),
          '#required' => TRUE,
        ),
        'myfax_email' => array(
          '#type'  => 'textfield',
          '#title' => 'MyFax account \'from\' email address',
          '#default_value' => connect_node_options( $parent->nid, 'myfax_email' ),
          '#required' => TRUE,
        ),
        'myfax_password' => array(
          '#type'  => 'textfield',
          '#title' => 'MyFax password',
          '#default_value' => connect_node_options( $parent->nid, 'myfax_password' ),
        ),
      );
      $base['parent'] = array(
        'fax_subject' => 'Fax subject',
        'fax_body'    => 'Fax body',
      );
      $base['child']  = array(
        'fax_lookup_result' => 'MyFax (lookup) success/failure message',
        'fax_from'   => 'Fax \'from\' name',
      );

      // add requirements from lookup type
      $lookup = connect_target_lookup('requires', $parent, $child, 'myfax');
      if (is_array($lookup)) {
        foreach( array('variables','parent','child') as $key ) {
          if (isset($lookup[$key])) {
            $return[$key] = $base[$key]+$lookup[$key];
          } else {
            $return[$key] = $base[$key];
          }
        }
      } else {
        $return = $base;
        drupal_set_message('You have not selected a lookup type. Please do so and then adjust any new settings that are required.');
      }
      return $return;
      break;

    case 'validate' :
      if ($target != 'child') {
        return;
      }
      $name = connect_value( 'fax_from', $parent, $child, 'child' );
      if ( empty($name) ) {
        form_set_error('', 'Please enter your name.');
      }
      // call validation from lookup type
      connect_target_lookup('validate', $parent, $child, 'myfax');
      break;

    case 'insert' :
      if ($target != 'child') {
        return;
      }
      require_once( drupal_get_path('module','connect') . '/connect_lookup.php');
      $target  = connect_target_lookup( 'lookup', $parent, $child, 'myfax' );
      if ($target && isset($target['fax']) && strlen($target['fax']) >= 10) {

        $fax = array();
        $fax['from_name'] = connect_value( 'fax_from', $parent, $child, 'child' );
        $fax['from_mail'] = connect_node_options( $parent->nid, 'myfax_email' );
        $fax['to_fax']    = $target['fax'];
        $fax['to_name']   = $target['name'];
        $fax['body']      = connect_value( 'fax_body', $parent, $child, 'parent' );
        $fax['password']  = connect_node_options( $parent->nid, 'myfax_password' );
        $fax['subject']   = connect_value( 'fax_subject', $parent, $child, 'parent' );

        $message = connect_myfax_prepare($fax);
        $active  = (connect_node_options($parent->nid, 'is_live') == 'yes');
        $result  = _connect_send_email($message, FALSE, $active);

      } else {
        $result = FALSE;
      }

      // save result
      $_SESSION['connect_'.$parent->nid.'_myfax_lookup_sent'] = $result ? $target['name'] : FALSE;
      $result = $result ? "Success" : "Failed";
      connect_value( 'fax_lookup_result', $parent, $child, 'child', $result);
      node_save($child);
      break;

    case 'status' :
      $message  = '';
      if ( isset($_SESSION['connect_'.$parent->nid.'_myfax_lookup_sent']) ) {
        $message = $_SESSION['connect_'.$parent->nid.'_myfax_lookup_sent'] ? 'Your fax message was sent to ' .$_SESSION['connect_'.$parent->nid.'_myfax_lookup_sent'] : 'Sorry, but there was a problem sending the message. The problem will be investigated, and your message will be sent when the problem is resolved.';
        unset($_SESSION['connect_'.$parent->nid.'_myfax_lookup_sent']);
      }
      return array(
        'status' => $message
      );
      break;
      break;
  }
}


/**** Utils ****/

/**
 *  turn fax information into an email message for the myfax service
 */
function connect_myfax_prepare($fax) {
  $faxNumber = preg_replace('/[^0-9]/','',$fax['to_fax']);
  if (strlen($faxNumber) < 10) {
    return FALSE;
  } elseif (strlen($faxNumber)==10) {
     // add default country code
    $faxNumber = '1'.$faxNumber;
  }

  $body = '';
  if (!empty($fax['password'])) {
    $body .= "$fax[password]\r\n \r\n";
  }
  $body .= "To: $fax[to_name] ($fax[to_fax])\r\n";
  $body .= "From: $fax[from_name]\r\n";
  $body .= "Subject: $fax[subject]\r\n\r\n";
  $body .= "$fax[body]\r\n\r\n$fax[from_name]";

  $message->to      = "$faxNumber@myfax.com";
  $message->from    = "$fax[from_name] <$fax[from_mail]>";
  $message->subject = $fax['to_name'];
  $message->body    = $body;

  return $message;
}

/*
 *   send mail or present debug info
 */
function _connect_send_email(&$message, $send_html = FALSE, $send_mail = FALSE) {
  if (!$send_mail) {
    foreach ($message as $key=>$value) {
      drupal_set_message("$key : " . htmlentities($value));
    }
    return TRUE;
  }
  if ( !empty($message->subject) &&
       !empty($message->body) &&
       _connect_valid_email($message->to) &&
       _connect_valid_email($message->from) ) {

    if ($send_html && module_exists('mimemail')) {
      return mimemail($message->from, $message->to, $message->subject, $message->body);
    }
    else {
      return drupal_mail('connect_send_email', $message->to, $message->subject, $message->body, $message->from);
    }
  } else {
    return false;
  }
}

/**
 *  validation wrapper that allows 'name <address>' format email addresses
 */
function _connect_valid_email( $address ) {
  $regex = '/(.*)\s?\<(.*)\>/';
  $expanded_format = preg_match($regex, $address, $matches);
  if ($expanded_format) {
    $address = $matches[2];
  }
  return valid_email_address($address);
}


// the civicrm API doesn't produce very helpful output
// so many of these have to be set manually
function connect_civicrm_properties_options() {
  civicrm_initialize( TRUE );

  $return = array ( 'first_name' => 'First Name',
                    'middle_name' => 'Middle Name',
                    'last_name' => 'Last Name',
                    'home_URL' => 'URL',
                    'street_address' => 'Address',
                    'city' => 'City',
                    'state_province' => 'State/Province',
                    'postal_code' => 'Postal Code',
                    'email' => 'Email address',
                  );

  /*
  $get_types = array('Individual','Location');
  foreach ($get_types as $type) {
    $fields = crm_get_class_properties( $type, 'custom' );
    foreach ($fields as $field) {
      $return[$field['name']] = $field['description'];
    }
  }
  */

  // add the easy-to-manage CiviCRM custom fields (no enums)
  // TODO : use mapping custom_ID_html_type to handle different types of data
  /*
  if (class_exists('CRM_Core_DAO_CustomField')) {
    $DAO =& new CRM_Core_DAO_CustomField;
    $DAO->html_type = 'text';
    $DAO->find();
    while ( $DAO->fetch() ) {
      $return['custom_'.$DAO->id] = $DAO->label;
    }
  }
  */
  return $return;
}
